#include "src/Oscillator/Oscillator.h"
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

//---------------------------------
// Servo Settings 舵机设定
//---------------------------------

#define N_SERVOS 5 //五自由度舵机 0左脚 1右脚 2左腿 3右腿 4头

#define INTERVALTIME 10.0 //这是中断时间为10.0

int t = 620;  // 1280; //脉冲宽度？
double pausemillis = 0;
// ch0:RightFoot,ch1:LeftFoot,ch2:RightLeg,ch3:LeftLeg
// ch4:NeckV
int servoMin[N_SERVOS]  = {150, 150, 150, 150, 150}; //舵机的最小输出
int servoMax[N_SERVOS]  = {500, 500, 500, 500, 500}; //舵机的最大输出
int servoTrim[N_SERVOS] = {10, 0, 0, -10, -10};      //
int servoInit[N_SERVOS] = {90, 90, 90, 90, 125};     //舵机初始化
int servoTest[N_SERVOS] = {80, 80, 80, 80, 115};     //舵机测试

#define NECKV_CH 4

Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(0x40);
Oscillator *servo[N_SERVOS];

bool maintenanceMode = true;

void setup() {
   Serial.begin(9600);
   Serial.println("5 channel Servo test!");
   
   Wire.begin();
   pwm.begin();//启动舵机
   pwm.setPWMFreq(50);  //模拟伺服在60赫兹更新下运行

   //-----------舵机初始化---------------------------
    for (int i = 0; i < N_SERVOS; i ++) {
     servo[i] = new Oscillator(&pwm, i, servoMin[i], servoMax[i], servoTrim[i]);
    }
    Serial.println("Servo init");
    for (int i = 0; i < N_SERVOS; i++) {
      servo[i]->setPosition(servoTest[i]);
    }
    delay(1000);
    for (int i = 0; i < N_SERVOS; i++) {
     servo[i]->setPosition(servoInit[i]);
    }
    
    //---------整机初始化完成，舵机休眠-----------
}

void loop() {
  action1();
}

//------------action1第一套编舞------------------------
void action1() {
  displayStatus("Walk");
  walk(2, t * 3);
  displayStatus("Back");
  backyard(2, t * 3);
  displayStatus("upDown");
  upDown(3, t * 2);
  displayStatus("MWalkR");
  moonWalkRight(1, t*2);
  moonWalkRight(2, t);
  displayStatus("MWalkL");
  moonWalkLeft(1, t*2);
  moonWalkLeft(2, t);
  displayStatus("swing");
  swing(2, t);
}
//---------------输出显示---------------------
void displayStatus(String param) {

 // Serial.print(param + "    ");

}
//-------oscillate函数----------------------
void oscillate(int A[N_SERVOS], int O[N_SERVOS], int T, double phase_diff[N_SERVOS]) {
  for (int i = 0; i < N_SERVOS; i++) {
    servo[i]->setO(O[i]);
    servo[i]->setA(A[i]);
    servo[i]->setT(T);
    servo[i]->setPh(phase_diff[i]);
  }


  double ref = millis();
  for (double x = ref; x < T + ref; x = millis()) {
    for (int i = 0; i < N_SERVOS; i++) {
      servo[i]->refresh();
    }
  }

 for (int i=0; i< N_SERVOS; i++) {
   servo[i]->reset();
    servo[i]->setPosition(90);
 }

}
//-------------------moveNServos函数------------------------
unsigned long final_time;
unsigned long interval_time;
int oneTime;
int iteration;
float increment[N_SERVOS];
int oldPosition[N_SERVOS] = {90, 90, 90, 90, 90};

void moveNServos(int time, int  newPosition[]) {
  for (int i = 0; i < N_SERVOS; i++)  increment[i] = ((newPosition[i]) - oldPosition[i]) / (time / INTERVALTIME);

  final_time =  millis() + time;

  iteration = 1;
  while (millis() < final_time) { //Javi del futuro cambia esto
    interval_time = millis() + INTERVALTIME;

    oneTime = 0;
    while (millis() < interval_time) {
      if (oneTime < 1) {
        for (int i = 0; i < N_SERVOS; i++) {
          servo[i]->setPosition(oldPosition[i] + (iteration * increment[i]));
        }
        iteration++;
        oneTime++;
      }
    }
  }

  for (int i = 0; i < N_SERVOS; i++) {
    oldPosition[i] = newPosition[i];
  }
}
//-------------------------------------------------
// Motions 机器人的基本动作
//-------------------------------------------------
void walk(int steps, int T) {
  int A[N_SERVOS] = { 15, 15, 30, 30, 25};
  int O[N_SERVOS] = { 0, 0, 0, 0, 0};
  double phase_diff[N_SERVOS] = { DEG2RAD(0), DEG2RAD(0), DEG2RAD(90), DEG2RAD(90), DEG2RAD(135) };

  for (int i = 0; i < steps; i++) {
    oscillate(A, O, T, phase_diff);
  }
}

void backyard(int steps, int T) {
  int A[N_SERVOS] = {15, 15, 30, 30, 25};
  int O[N_SERVOS] = {0, 0, 0, 0, 0};
  double phase_diff[N_SERVOS] = {DEG2RAD(0), DEG2RAD(0), DEG2RAD(-90), DEG2RAD(-90), DEG2RAD(135)};

  for (int i = 0; i < steps; i++) oscillate(A, O, T, phase_diff);
}


void upDown(int steps, int tempo) {
  int move1[N_SERVOS] = {50, 130, 90, 90, 115};
  int move2[N_SERVOS] = {90, 90, 90, 90, 90};

  for (int x = 0; x < steps; x++) {
    pausemillis = millis();
    moveNServos(tempo * 0.2, move1);
    delay(tempo * 0.4);
    moveNServos(tempo * 0.2, move2);
    while (millis() < (pausemillis + tempo));
  }
}

void moonWalkRight(int steps, int T) {
  Serial.println("moonWalkRight");
  int A[N_SERVOS] = {15, 15, 0, 0, 0};
  int O[N_SERVOS] = {5 , 5, 0, 0, 0};
  double phase_diff[N_SERVOS] = {DEG2RAD(0), DEG2RAD(180 + 120), DEG2RAD(90), DEG2RAD(90), DEG2RAD(90)};

  for (int i = 0; i < steps; i++) oscillate(A, O, T, phase_diff);
}

void moonWalkLeft(int steps, int T) {
  Serial.println("moonWalkLeft");
  int A[N_SERVOS] = {15, 15, 0, 0, 0};
  int O[N_SERVOS] = {5, 5, 0, 0, 0};
  double phase_diff[6] = {DEG2RAD(0), DEG2RAD(180 - 120), DEG2RAD(90), DEG2RAD(90), DEG2RAD(90)};

  for (int i = 0; i < steps; i++) oscillate(A, O, T, phase_diff);
}

void swing(int steps, int T) {
  int A[N_SERVOS] = {25, 25, 0, 0, 0};
  int O[N_SERVOS] = {15, 15, 0, 0, 0};
  double phase_diff[N_SERVOS] = {DEG2RAD(0), DEG2RAD(0), DEG2RAD(90), DEG2RAD(90), DEG2RAD(90)};

  for (int i = 0; i < steps; i++) oscillate(A, O, T, phase_diff);
}

void headSwing(int steps, int tempo) {
  int move1[N_SERVOS] = {90, 90, 90, 90, 90};
  int move2[N_SERVOS] = {90, 90, 90, 90, 45};

  for (int x = 0; x < steps; x++) {
    pausemillis = millis();
    moveNServos(tempo * 0.2, move1);
    delay(tempo * 0.4);
    moveNServos(tempo * 0.2, move2);
    while (millis() < (pausemillis + tempo));
  }
}

void headSwing2(int steps, int T) {
  int A[N_SERVOS] = {0, 0, 0, 0, 45};
  int O[N_SERVOS] = {0, 0, 0, 0, 0};
  double phase_diff[N_SERVOS] = {DEG2RAD(90), DEG2RAD(90), DEG2RAD(90), DEG2RAD(90), DEG2RAD(135) };

  for (int i = 0; i < steps; i++) oscillate(A, O, T, phase_diff);
}
